<?php
// $Id: node_widget.form.inc,v 1.3 2010/02/18 04:09:33 antoniodemarco Exp $

/**
 * Implementation of hook_forms()
 */
function node_widget_forms() {
  $forms = array();
  if ($types = node_get_types()) {
    foreach (array_keys($types) as $type) {
      $forms[$type .'_node_widget_form']['callback'] = 'node_widget_form';
    }
  }
  return $forms;
}

/**
 * Generate the node add/edit form array.
 */
function node_widget_form($form_state, $node, $element = NULL) {
  global $user;

  $form_id = $node->type .'_node_widget_form';

  // Handle node object
  if (isset($form_state['node'])) {
    $node = $form_state['node'] + (array)$node;
  }
  $node = (object)$node;
  foreach (array('body', 'title', 'format') as $key) {
    if (!isset($node->$key)) {
      $node->$key = NULL;
    }
  }
  if (!$node->nid) {
    module_load_include('inc', 'node', 'node.pages');
    node_object_prepare($node);
  }

  // Add necessary information to $form array.
  // #type should be set to 'form' but it would cause problem running form_builder(),
  // the node widget form will be built during the main #after_built callback.
  // @see: node_widget_node_form_after_build()
  // With #printed = TRUE the form element itself will be not rendered.
  $form['form_id'] = array('#type' => 'hidden', '#value' => $form_id, '#id' => form_clean_id("edit-$form_id"),);
  $form['#type'] = NULL;
  $form['#node'] = $node;
  $form['#printed'] = TRUE;
  $form['#validate'][] = 'node_widget_form_validate';
  $form['#theme'][] = $form_id;
  $form['#theme'][] = 'node_widget_form';

  // Add basic node information.
  foreach (array('nid', 'vid', 'uid', 'created', 'type', 'language', 'changed') as $key) {
    $form[$key] = array('#type' => 'hidden', '#value' => isset($node->$key) ? $node->$key : NULL,);
  }

  // Adding form specific #validate and #submit callback.
  if (!isset($form['#validate'])) {
    if (function_exists($form_id .'_validate')) {
      $form['#validate'] = array($form_id .'_validate');
    }
  }
  if (!isset($form['#submit'])) {
    if (function_exists($form_id .'_submit')) {
      $form['#submit'] = array($form_id .'_submit');
    }
  }

  if ($node->nid) {
    $form['node_widget_link'] = array(
      '#type' => 'markup',
      '#value' => l(t('View'), 'node/'. $node->nid),
      '#prefix' => '<div class="node-widget-link">',
      '#suffix' => '</div>',
      '#weight' => -100,
    );
  }

  $form['remove'] = array(
    '#type' => 'checkbox',
    '#title' => t('Remove'),
    '#prefix' => '<div class="node-widget-remove">',
    '#suffix' => '</div>',
    '#default_value' => FALSE,
    '#weight' => -99,
  );

  // @TODO: inherit from parent node a part from promote and sticky

  // Run hook_node_widget_form_alter allowing modules
  // to add their own fields (i.e. CCK, Taxonomy, etc...)
  $data = &$form;
  $data['__drupal_alter_by_ref'] = array(&$form_state);
  drupal_alter('node_widget_form_'. $form_id, $data);
  $data['__drupal_alter_by_ref'] = array(&$form_state);
  drupal_alter('node_widget_form', $data, $form_id);

  return $form;
}

/**
 * Implementation of hook_form_alter()
 */
function node_widget_form_alter(&$form, &$form_state, $form_id) {
  // Modify field widget settings form: only one content type
  // can be referenced and advanced node view rendering is not allowed.
  if ($form_id == 'content_field_edit_form' && $form['#field']['widget']['type'] = 'node_widget_node_form') {
    unset($form['field']['advanced']);
  }
}

/**
 * Implementation of #after_build callback.
 * Used to turn off validation to node widget form elements.
 *
 * @see: node_widget_widget()
 */
function node_widget_node_form_after_build(&$form, &$form_state) {
  foreach ($form['#field_info'] as $field_name => $field) {
    if ($field['type'] == 'nodereference' && $field['widget']['type'] == 'node_widget_node_form') {
      // Disable element validation, ti will take place in node_widget_element_validate()

      $group = _node_widget_get_group($field);
      foreach (_node_widget_get_element_children($form, $field) as $key) {
        if (is_int($key)) {
          if ($group) {
            _node_widget_form_builder($form[$group][$field_name][$key]['node_widget'], $form_state['values'][$field_name][$key]['node_widget']);
            _node_widget_toggle_validation($form[$group][$field_name][$key]['node_widget'], FALSE);
          }
          else {
            _node_widget_form_builder($form[$field_name][$key]['node_widget'], $form_state['values'][$field_name][$key]['node_widget']);
            _node_widget_toggle_validation($form[$field_name][$key]['node_widget'], FALSE);
          }
        }
      }

    }
  }
  return $form;
}

/**
 * Implementation of node form #submit callback.
 */
function node_widget_node_form_submit(&$form, &$form_state) {
  global $user;

  foreach (content_fields() as $field_name => $field) {
    if ($form['type']['#value'] == $field['type_name'] && $field['type'] == 'nodereference' && $field['widget']['type'] == 'node_widget_node_form') {

      $group = _node_widget_get_group($field);
      foreach (_node_widget_get_element_children($form, $field) as $key) {
        if (is_numeric($key) && in_array('node_form_submit', $form_state['clicked_button']['#submit'])) {
          $_form = $group ? $form[$group][$field_name][$key]['node_widget'] : $form[$field_name][$key]['node_widget'];
          $_form_state['values'] = $form_state['values'][$field_name][$key]['node_widget'];
          $_form_state['storage'] = NULL;
          $_form_state['submitted'] = TRUE;
          $_form_state['clicked_button'] = $form_state['clicked_button'];
          $_form_state['#field_info'] = $_form['#field_info'];

          if ($_form_state['values']['remove'] && $_form_state['values']['nid']) {
            node_delete($_form_state['values']['nid']);
            unset($form_state['values'][$field_name][$key]);
          } elseif (!_node_widget_is_empty($_form, $_form_state)) {
            node_form_submit($_form, $_form_state);
            $form_state['values'][$field_name][$key]['nid'] = $_form_state['nid'];
          }

        }
      }
    }
  }
}

/**
 * Implementation of node form #validate callback.
 */
function node_widget_node_form_validate(&$form, &$form_state) {
  global $user;

  foreach (content_fields() as $field_name => $field) {
    if ($form['type']['#value'] == $field['type_name'] && $field['type'] == 'nodereference' && $field['widget']['type'] == 'node_widget_node_form') {

      $group = _node_widget_get_group($field);
      foreach (_node_widget_get_element_children($form, $field) as $key) {
        if (is_numeric($key)) {

          if ($group) {
            $element = &$form[$group][$field_name][$key];
            $_form = $form[$group][$field_name][$key]['node_widget'];
          }
          else {
            $element = &$form[$field_name][$key];
            $_form = $form[$field_name][$key]['node_widget'];
          }

          $delta = $element['#delta'];
          $_form_id = $element['node_widget']['form_id']['#value'] .'_'. $delta;
          $_form_state['values'] = &$form_state['values'][$field_name][$key]['node_widget'];
          $_form_state['values']['uid'] = $user->uid;
          $_form_state['values']['name'] = $user->name;
          $_form_state['clicked_button'] = $form_state['clicked_button'];
          $_form_state['#field_info'] = $_form['#field_info'];

          $_form['#post'] = $element['#post'][$element['#field_name']][$delta]['node_widget'];
          foreach ($_form['#field_info'] as $_name => $field) {
            $_form[$_name]['#post'] = &$_form['#post'];
            foreach (element_children($_form[$_name]) as $key) {
              $_form[$_name][$key]['#post'] = &$_form['#post'];
            }
          }

          _node_widget_reset_parents($_form);
          if (!_node_widget_is_empty($_form, $_form_state)) {
            _node_widget_toggle_validation($_form, TRUE);
            drupal_validate_form($_form_id, $_form, $_form_state);
            node_validate($_form_state['values'], $_form);
            _node_widget_fix_error_messages($element);
          }
        }
      }
    }
  }
}

/**
 * Recursively reset #parents array to original value.
 */
function _node_widget_reset_parents(&$elements) {
  foreach (element_children($elements) as $key) {
    if (isset($elements[$key]) && $elements[$key]) {
      _node_widget_reset_parents($elements[$key]);
    }
  }
  if (is_array($elements['#parents']) && in_array('node_widget', $elements['#parents'])) {
    // @todo: that's not nice... make it better.
    $elements['#parents'] = array_slice($elements['#parents'], 3);
  }
}


/**
 * Fix Drupal error messages by cleaning duplicates
 * and highlighting the right form fields
 *
 * @param $element
 *    Node widget form element.
 */
function _node_widget_fix_error_messages(&$element) {
  static $errors_cache = array();
  $children = element_children($element['node_widget']);
  $delta = $element['#delta'];
  $field_name = $element['#field_name'];

  // Fix error messages.
  $processed = array();
  $errors = form_get_errors() ? form_get_errors() : array();
  form_set_error(NULL, NULL, TRUE);
  $diff = array_diff_key($errors, $errors_cache);
  foreach ($diff as $key => $message) {
    $name = strstr($key, '][') === FALSE ? $key : array_shift(explode('][', $key));
    if (in_array($name, $children)) {
      $key = $field_name .']['. $delta .'][node_widget]['. $key;
    }
    $processed[$key] = $message;
  }
  $errors_cache = array_merge($processed, $errors_cache);
  foreach ($errors_cache as $key => $message) {
    form_set_error($key, $message);
  }

  // Remove duplicatete error messages
  if (is_array($_SESSION['messages']['error'])) {
    $duplicate = array();
    foreach ($_SESSION['messages']['error'] as $key => $message) {
      if (in_array($message, $duplicate)) {
        unset($_SESSION['messages']['error'][$key]);
      }
      $duplicate[] = $message;
    }
  }
}

/**
 * Toggle FAPI element validation by recursively setting #valideted to TRUE or FALSE.
 *
 * @param $elements
 *    Array of FAPI elements
 * @param $value
 *    TRUE or FALSE
 */
function _node_widget_toggle_validation(&$elements, $value = TRUE) {
  foreach (element_children($elements) as $key) {
    if (isset($elements[$key]) && $elements[$key]) {
      _node_widget_toggle_validation($elements[$key], $value);
    }
  }
  if ($elements['#needs_validation']) {
    $elements['#validated'] = !$value;
  }
}

/**
 * Check if a node widget form contains an empty node.
 *
 * @param $form
 * @param $form_state
 */
function _node_widget_is_empty(&$form, &$form_state) {
  $result = module_invoke_all('node_widget_is_empty', $form, $form_state);
  if (is_array($result)) {
    foreach ($result as $empty) {
      $return = isset($return) ? ($return && $empty) : $empty;
    }
  }
  return $return;
}

/**
 * Build a node widget form.
 */
function _node_widget_form_builder(&$form, &$form_state) {
  $form['#type'] = 'form';
  $form_id = $form['#node_type'] .'_node_widget_form';
  $form = form_builder($form_id, $form, $form_state);
}

/**
 * Get fieldgroup of a field, if any.
 *
 * @param $field
 *    Field object
 */
function _node_widget_get_group($field) {
  $group = NULL;
  if (module_exists('fieldgroup')) {
    $group = fieldgroup_get_group($field['type_name'], $field['field_name']);
  }
  return $group;
}

/**
 * Get element children for a specified node widget field
 *
 * @param $field
 *    Field object
 */
function _node_widget_get_element_children(&$form, $field) {
  $group = NULL;
  if (module_exists('fieldgroup') && $group = fieldgroup_get_group($field['type_name'], $field['field_name'])) {
    $elements = element_children($form[$group][$field['field_name']]);
  }
  else {
    $elements = element_children($form[$field['field_name']]);
  }
  return $elements;
}